<?php
/**
 * Wicked_Driver:: defines an API for implementing storage backends for Wicked.
 *
 * $Horde: wicked/lib/Driver.php,v 1.31 2005/01/03 14:35:46 jan Exp $
 *
 * Copyright 2003-2005 Tyler Colbert <tyler@colberts.us>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Tyler Colbert <tyler-hordeml@colberts.us>
 * @version $Revision: 1.31 $
 * @since   Wicked 0.1
 * @package Wicked
 */
class Wicked_Driver {

    /**
     * Hash containing connection parameters.
     *
     * @var array $_params
     */
    var $_params = array();

    /**
     * Constructs a new Wicked driver object.
     *
     * @param array  $params    A hash containing connection parameters.
     */
    function Wicked_Driver($params = array())
    {
        $this->_params = $params;
    }

    /**
     * Create a new Wicked page. If there is a standard page in
     * scripts/data for this page, load it. Otherwise, create a blank
     * page and let the user know they are creating the page.
     *
     * @param string $pagename  The name of the page to load.
     */
    function loadPage($pagename, $template)
    {
        if ($GLOBALS['browser']->isRobot()) {
            return PEAR::raiseError(_("Robots are not allowed to create pages."));
        }

        global $notification;

        $pagefile = WICKED_BASE . '/scripts/data/' . basename($pagename);
        if (!empty($template)) {
            $tpage = $this->retrieveByName($template);
            if (is_a($tpage, 'PEAR_Error')) {
                $notification->push(sprintf(_("Error loading page template: %s"), $tpage->getMessage()), 'horde.error');
                return $tpage;
            }
            $notification->push(sprintf(_("Created %s. Edit this page to set up initial content."), $pagename), 'horde.message');
            $text = $tpage['page_text'];
        } elseif (file_exists($pagefile)) {
            $fp = @fopen($pagefile, 'r');
            $text = fread($fp, filesize($pagefile));
            fclose($fp);
        } else {
            $notification->push(sprintf(_("Created %s. Edit this page to set up initial content."), $pagename), 'horde.message');
            $text = '';
        }

        $result = $this->newPage($pagename, $text);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        /* Call retrieveByName() with the optional $autocreate
         * parameter set to false so that in case something goes
         * wrong, we don't create an infinite loop between it and this
         * function. */
        $new_page = $this->retrieveByName($pagename, false);

        if (!empty($GLOBALS['conf']['wicked']['notify_address'])) {
            /* Send notification. */
            require_once 'Horde/MIME.php';

            $headers = array('Subject' => MIME::encode('[' . $GLOBALS['registry']->get('name') . '] created: ' . $new_page['page_name'], NLS::getCharset()));

            if (Auth::getAuth()) {
                $message = Auth::getAuth();
            } else {
                $message = 'guest [' . $_SERVER['REMOTE_ADDR'] . ']';
            }
            $lc_time = setlocale(LC_TIME, 'C');
            $message .= '  ' . date('r') . "\n\n";
            setlocale(LC_TIME, $lc_time);

            $url = Horde::applicationUrl('display.php', true, -1);
            $url = Util::addParameter($url, 'page', $new_page['page_name']);
            $message .= "Created page: $url\n";

            Wicked::mail($message, $headers);
        }

        return $new_page;
    }

    /**
     * Retrieves the page of a particular name from the database.
     *
     * @param string $pagename     The name of the page to retrieve
     * @param boolean $autocreate  (optional) Try to create the page
     *                             from a standard page if we can't load it
     *                             from the backend? Defaults to false.
     *
     * @return mixed  True on success, PEAR_Error on failure.
     */
    function retrieveByName($pagename, $autocreate = false)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    /**
     * Retrieve a historic version of a page.
     *
     * @abstract
     * @param string $pagename      The name of the page to retrieve.
     * @param string $version       The version to retrieve.
     * @return mixed The page hash, or PEAR_Error on failure.
     */
    function retrieveHistory($pagename, $version)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    /**
     * Log a hit to $pagename.
     *
     * @param string $pagename  The page that was viewed.
     *
     * @return mixed  True or PEAR_Error on failure.
     */
    function logPageView($pagename)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    function lock($pagename, $locked)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    /**
     * Create a new page.
     *
     * @abstract
     * @param string $pagename          The new page's name.
     * @param string $text              The new page's text.
     * @return mixed True, or PEAR_Error on failure.
     */
    function newPage($pagename, $text)
    {
        return PEAR::raiseError(_("Not implemented."));
    }

    function updateText($pagename, $text, $changelog, $minorchange)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    function renamePage($pagename, $newname)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    function getPage()
    {
        return array();
    }

    function getPages()
    {
        return array();
    }

    function pageExists($pagename)
    {
        return in_array($pagename, $this->getPages());
    }

    function getAllPages()
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    function getHistory($pagename)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    function getRecentChanges()
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    /**
     * Return the most popular pages.
     *
     * @abstract
     * @param optional integer $limit       The number of most popular pages
     *                                      to return.  (Default: 10).
     * @return mixed An array of pages, or PEAR_Error on failure.
     */
    function mostPopular($limit = 10)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    /**
     * Return the least popular pages.
     *
     * @abstract
     * @param optional integer $limit       The number of least popular pages
     *                                      to return.  (Default: 10).
     * @return mixed An array of pages, or PEAR_Error on failure.
     */
    function leastPopular($limit = 10)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    /**
     * Find pages with matches in text or title.
     *
     * @abstract
     * @param string $searchtext        The search expression (Google-like).
     * @return mixed An array of pages, or PEAR_Error on failure.
     */
    function searchText($searchtext)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    function getBackLinks($pagename)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    function getLikePages($pagename)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    /**
     * Retrieve data on files attached to a page.
     *
     * @param string $pagename              This is the name of the page for
     *                                      which we'd like to find attached
     *                                      files.
     * @param optional bool $allversions    Whether to include all versions.
     *                                      If false or omitted, only the most
     *                                      recent version of each attachment
     *                                      is returned.
     * @return mixed An array of key/value arrays describing the attached
     *      files or a PEAR_Error:: instance on failure.
     */
    function getAttachedFiles($pagename, $allversions = false)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    /**
     * Attach a file to a page or update an attachment.
     *
     * @param array $file  This is a key/value array describing the attachment:
     *
     *           'page_name' =>        This is the name of the page to which
     *                                 we would like to attach the file.
     *           'attachment_name' =>  This is the filename of the attachment.
     *           'minor' =>            This is a boolean which indicates
     *                                 whether this is a minor version update.
     *           'change_log' =>       A change log entry for this attach or
     *                                 update operation.  (Optional)
     *           'change_author' =>    The user uploading this file.  If not
     *                                 present, the currenlty logged-in user
     *                                 is assumed.
     *
     * @param string $data This is the contents of the file to be attached.
     * @return mixed true or PEAR_Error:: instance on failure.
     */
    function attachFile($file, $data)
    {
        global $conf;

        require_once 'VFS.php';
        $vfs = &VFS::singleton($conf['vfs']['type'],
                               Horde::getDriverConfig('vfs'));
        if (is_a($vfs, 'PEAR_Error')) {
            return $vfs;
        }

        if (!isset($file['change_author'])) {
            $file['change_author'] = Auth::getAuth();
        }

        $res = $this->_attachFile($file);
        if (is_a($res, 'PEAR_Error')) {
            return $res;
        }

        /* We encode the path quoted printable so we won't get any nasty
         * characters the filesystem might reject. */
        $path = WICKED_VFS_ATTACH_PATH . '/' . preg_replace('/([^a-zA-Z_0-9 ])/e', '"=" . str_pad(dechex(ord(\'\\1\')), 2, \'0\', STR_PAD_LEFT)', $file['page_name']);
        $name = sprintf("%s;%s", $file['attachment_name'], $res);
        return $vfs->writeData($path, $name, $data, true);
    }

    /**
     * Handle the driver-specific portion of attaching a file.
     *
     * Wicked_Driver::attachFile() calls down to this method for the driver-
     * specific portion, and then uses VFS to store the attachment.
     *
     * @access protected
     *
     * @param array $file               See Wicked_Driver::attachFile().
     * @return mixed The new version of the file attached, or a PEAR_Error::
     *      instance on failure.
     */
    function _attachFile($file)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    /**
     * Retrieve the contents of an attachment.
     *
     * @param string $pagename          This is the name of the page to which
     *                                  the file is attached.
     * @param string $filename          This is the name of the attachment.
     * @param string $version           This is the version of the attachment.
     * @return mixed A string containing the file's contents or a PEAR_Error::
     *      instance on error.
     */
    function getAttachmentContents($pagename, $filename, $version)
    {
        global $conf;

        require_once 'VFS.php';
        $vfs = &VFS::singleton($conf['vfs']['type'],
                               Horde::getDriverConfig('vfs'));
        if (is_a($vfs, 'PEAR_Error')) {
            return $vfs;
        }

        $path = WICKED_VFS_ATTACH_PATH . '/' . preg_replace('/([^a-zA-Z_0-9 ])/e', '"=" . str_pad(dechex(ord(\'\\1\')), 2, \'0\', STR_PAD_LEFT)', $pagename);
        $name = sprintf("%s;%s", $filename, $version);

        return $vfs->read($path, $name);
    }

    function removeVersion($pagename, $version)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    function removeAllVersions($pagename)
    {
        $GLOBALS['notification']->push(sprintf(_("Deleted %s."), $pagename), 'horde.message');

        if (!empty($GLOBALS['conf']['wicked']['notify_address'])) {
            /* Send notification. */
            require_once 'Horde/MIME.php';

            $headers = array('Subject' => MIME::encode('[' . $GLOBALS['registry']->get('name') . '] deleted: ' . $pagename, NLS::getCharset()));

            if (Auth::getAuth()) {
                $message = Auth::getAuth();
            } else {
                $message = 'guest [' . $_SERVER['REMOTE_ADDR'] . ']';
            }
            $lc_time = setlocale(LC_TIME, 'C');
            $message .= '  ' . date('r') . "\n\n";
            setlocale(LC_TIME, $lc_time);

            $message .= "Deleted page: $pagename\n";

            Wicked::mail($message, $headers);
        }
    }

    function searchTitles($searchtext)
    {
        return PEAR::raiseError(_("Unsupported"));
    }

    /**
     * Returns the charset used by the backend.
     *
     * @return string  The backend's charset
     */
    function getCharset()
    {
        return NLS::getCharset();
    }

    /**
     * Attempts to return a concrete Wicked_Driver instance based on
     * $driver.
     *
     * @param string $driver  The type of the concrete Wicked_Driver subclass
     *                        to return.  The class name is based on the storage
     *                        driver ($driver).  The code is dynamically
     *                        included.
     *
     * @param array  $params  (optional) A hash containing any additional
     *                        configuration or connection parameters a
     *                        subclass might need.
     *
     * @return mixed  The newly created concrete Wicked_Driver instance, or
     *                false on an error.
     */
    function &factory($driver = null, $params = null)
    {
        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        $driver = basename($driver);

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        require_once dirname(__FILE__) . '/Driver/' . $driver . '.php';
        $class = 'Wicked_Driver_' . $driver;
        if (class_exists($class)) {
            return $wicked = &new $class($params);
        } else {
            return false;
        }
    }

    /**
     * Attempts to return a reference to a concrete Wicked_Driver
     * instance based on $driver. It will only create a new instance
     * if no Wicked_Driver instance with the same parameters currently
     * exists.
     *
     * This method must be invoked as: $var = &Wicked_Driver::singleton()
     *
     * @param string $driver  The type of concrete Wicked_Driver subclass
     *                        to return.  The is based on the storage
     *                        driver ($driver).  The code is dynamically
     *                        included.
     * @param array $params   (optional) A hash containing any additional
     *                        configuration or connection parameters a
     *                        subclass might need.
     *
     * @return mixed  The created concrete Wicked_Driver instance, or false
     *                on error.
     */
    function &singleton($driver = null, $params = null)
    {
        static $instances;

        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Wicked_Driver::factory($driver, $params);
        }

        return $instances[$signature];
    }

}
